﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls.Charting;
using Telerik.Windows.Controls.GridView;
using System.Linq;
using Telerik.Windows.Controls;

namespace SL_Chart_GridView_SynchronizedHovering
{
    public partial class MainPage : UserControl
    {
        private static string ChartItem_HiddenStateName = "Hidden";
        private static string ChartItem_NormalState = "Normal";

        private static string GridRow_MouseOverStateName = "MouseOver";
        private static string GridRow_NormalStateName = "Normal";

        public MainPage()
        {
            InitializeComponent();

            var products = new List<Product>() 
            {
                new Product(){ProductName="Keyboard", ProductPrice=39.99},
                new Product(){ProductName="Mouse", ProductPrice=19.99},
                new Product(){ProductName="Monitor", ProductPrice=39.99},
                new Product(){ProductName="Motherboard", ProductPrice=9.99},
            };

            this.radChart1.ItemsSource = products;
            this.radGridViewProducts.ItemsSource = products;

            this.radGridViewProducts.RowLoaded += radGridViewProducts_RowLoaded;
            this.radGridViewProducts.RowUnloaded += radGridViewProducts_RowUnloaded;

            this.radChart1.DefaultView.ChartArea.HoverChanged += ChartArea_HoverChanged;
        }

        void ChartArea_HoverChanged(object sender, ChartHoverChangedEventArgs e)
        {
            var allGridRows = this.GetAllGridRows();
            foreach (var gridRow in allGridRows)
            {
                VisualStateManager.GoToState(gridRow, GridRow_NormalStateName, false);
            }

            foreach (var dataPoint in e.HoveredItems)
            {
                var gridRow = this.FindGridRow(dataPoint.DataItem);
                if (gridRow != null)
                {
                    VisualStateManager.GoToState(gridRow, GridRow_MouseOverStateName, false);
                }
            }
        }

        private void radGridViewProducts_RowUnloaded(object sender, Telerik.Windows.Controls.GridView.RowUnloadedEventArgs e)
        {
            e.Row.MouseEnter -= Row_MouseEnter;
            e.Row.MouseLeave -= Row_MouseLeave;
        }

        private void radGridViewProducts_RowLoaded(object sender, Telerik.Windows.Controls.GridView.RowLoadedEventArgs e)
        {
            e.Row.MouseEnter += Row_MouseEnter;
            e.Row.MouseLeave += Row_MouseLeave;
        }

        private void Row_MouseLeave(object sender, MouseEventArgs e)
        {
            UnhoverChartItems<Pie>();
            UnhoverChartItems<ChartLegendItem>();
            UnhoverChartItems<SeriesItemLabel>();
        }

        private void Row_MouseEnter(object sender, MouseEventArgs e)
        {
            var rowItem = sender as GridViewRowItem;

            if (rowItem != null && rowItem.Item != null)
            {
                HoverChartItem<Pie>(rowItem.Item);
                HoverChartItem<ChartLegendItem>(rowItem.Item);
                HoverChartItem<SeriesItemLabel>(rowItem.Item);
            }
        }

        private void HoverChartItem<T>(object dataItem) where T : Control
        {
            T hoveredItem = this.FindChartItem<T>(dataItem);

            var allItems = this.radChart1.ChildrenOfType<T>();
            foreach (var currentItem in allItems)
            {
                string state = hoveredItem == currentItem ? ChartItem_NormalState : ChartItem_HiddenStateName;
                VisualStateManager.GoToState(currentItem, state, false);
            }
        }

        private void UnhoverChartItems<T>() where T : Control
        {
            var allItems = this.radChart1.ChildrenOfType<T>();
            foreach (var item in allItems)
            {
                VisualStateManager.GoToState(item, ChartItem_NormalState, false);
            }
        }

        private IEnumerable<GridViewRow> GetAllGridRows()
        {
            return this.radGridViewProducts.ChildrenOfType<GridViewRow>();
        }

        private GridViewRow FindGridRow(object dataItem)
        {
            IEnumerable<GridViewRow> gridRows = this.radGridViewProducts.ChildrenOfType<GridViewRow>();

            foreach (GridViewRow gridRow in gridRows)
            {
                RadRowItem rowItem = gridRow as RadRowItem;

                if (rowItem != null)
                {
                    if (rowItem.Item != null && rowItem.Item == dataItem)
                    {
                        return gridRow;
                    }
                }
            }

            return null;
        }

        private T FindChartItem<T>(object dataItem) where T : Control
        {
            foreach (var item in this.radChart1.ChildrenOfType<T>())
            {
                DataPoint dataPoint = item.DataContext as DataPoint;
                if (dataPoint != null && dataPoint.DataItem == dataItem)
                {
                    return item;
                }
            }

            return null;
        }

    }
}
